#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_wifi.h"
#include "esp_event_loop.h"
#include "esp_log.h"
#include "nvs_flash.h"

#include "lwip/err.h"
#include "lwip/sys.h"

static EventGroupHandle_t wifi_event_group;

const int WIFI_CONNECTED_BIT = BIT0;
const int WIFI_DISCONNECTED_BIT = BIT1;
const int WIFI_STARTED_BIT = BIT2;

static const char *TAG = "simple wifi";

static esp_err_t event_handler(void *ctx, system_event_t *event)
{
    switch(event->event_id) {
    case SYSTEM_EVENT_STA_START:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_START");
        xEventGroupSetBits(wifi_event_group, WIFI_STARTED_BIT);
        break;
    case SYSTEM_EVENT_STA_GOT_IP:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_GOT_IP: %s",
                 ip4addr_ntoa(&event->event_info.got_ip.ip_info.ip));
        xEventGroupClearBits(wifi_event_group, WIFI_DISCONNECTED_BIT);
        xEventGroupSetBits(wifi_event_group, WIFI_CONNECTED_BIT);
        break;
    case SYSTEM_EVENT_AP_STACONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_AP_STACONNECTED:"MACSTR" join, AID=%d",
                 MAC2STR(event->event_info.sta_connected.mac),
                 event->event_info.sta_connected.aid);
        break;
    case SYSTEM_EVENT_AP_STADISCONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_AP_STADISCONNECTED:"MACSTR"leave, AID=%d",
                 MAC2STR(event->event_info.sta_disconnected.mac),
                 event->event_info.sta_disconnected.aid);
        break;
    case SYSTEM_EVENT_STA_DISCONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_DISCONNECTED: \"%s\"", event->event_info.disconnected.ssid);
        xEventGroupClearBits(wifi_event_group, WIFI_CONNECTED_BIT);
        xEventGroupSetBits(wifi_event_group, WIFI_DISCONNECTED_BIT);
        break;
    default:
        ESP_LOGI(TAG, "Unknown event: %d", event->event_id);
        break;
    }
    return ESP_OK;
}

void app_main()
{
    esp_err_t err = nvs_flash_init();
    if (err == ESP_ERR_NVS_NO_FREE_PAGES) {
      err = nvs_flash_erase();
      if (err == ESP_OK) {
        err = nvs_flash_init();
      }
    }
    ESP_ERROR_CHECK(err);


    wifi_event_group = xEventGroupCreate();

    tcpip_adapter_init();
    ESP_ERROR_CHECK(esp_event_loop_init(event_handler, NULL) );
    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));
    ESP_ERROR_CHECK(esp_wifi_set_storage(WIFI_STORAGE_RAM));
    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA) );
    ESP_ERROR_CHECK(esp_wifi_set_auto_connect(false));
    ESP_ERROR_CHECK(esp_wifi_start());

    xEventGroupWaitBits(wifi_event_group, WIFI_STARTED_BIT, false, true, portMAX_DELAY);

    xEventGroupClearBits(wifi_event_group, WIFI_CONNECTED_BIT);
    xEventGroupSetBits(wifi_event_group, WIFI_DISCONNECTED_BIT);

    int index = 0;
    while (1) {
      ESP_LOGI(TAG, "Wait for disconnect.");
      xEventGroupWaitBits(wifi_event_group, WIFI_DISCONNECTED_BIT, true, true, portMAX_DELAY);
      if (index == 0) {
        index = 1;
        wifi_config_t wifi_config = {
            .sta = {
                .ssid = CONFIG_ESP_WIFI_SSID1,
                .password = CONFIG_ESP_WIFI_PASSWORD1
            },
        };
        ESP_ERROR_CHECK(esp_wifi_set_config(ESP_IF_WIFI_STA, &wifi_config));
        ESP_LOGI(TAG, "Connect to %s.", wifi_config.sta.ssid);
        ESP_ERROR_CHECK(esp_wifi_connect());
      }
      else {
        index = 0;
        wifi_config_t wifi_config = {
            .sta = {
                .ssid = CONFIG_ESP_WIFI_SSID2,
                .password = CONFIG_ESP_WIFI_PASSWORD2
            },
        };
        ESP_ERROR_CHECK(esp_wifi_set_config(ESP_IF_WIFI_STA, &wifi_config));
        ESP_LOGI(TAG, "Connect to %s.", wifi_config.sta.ssid);
        ESP_ERROR_CHECK(esp_wifi_connect());
      }

      vTaskDelay(1000 / portTICK_PERIOD_MS);
    }
}
